/*
 * dvbhddevice.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: dvbhddevice.c 1.2 2010/01/01 15:00:53 kls Exp $
 */

#include <vdr/plugin.h>
#include <vdr/shutdown.h>
#include "dvbhdffdevice.h"
#include "menu.h"
#include "setup.h"

static const char *VERSION        = "0.0.4dag";
static const char *DESCRIPTION    = trNOOP("HD Full Featured DVB device");
static const char *MAINMENUENTRY  = "dvbhddevice";

class cPluginDvbhddevice : public cPlugin {
  bool mIsUserInactive;
public:
  cPluginDvbhddevice(void);
  virtual ~cPluginDvbhddevice();
  virtual const char *Version(void) { return VERSION; }
  virtual const char *Description(void) { return tr(DESCRIPTION); }
  virtual void MainThreadHook(void);
  virtual const char *MainMenuEntry(void);
  virtual cOsdObject *MainMenuAction(void);
  virtual cMenuSetupPage *SetupMenu(void);
  virtual bool SetupParse(const char *Name, const char *Value);
  virtual bool Initialize();
  };

cPluginDvbhddevice::cPluginDvbhddevice(void){}

cPluginDvbhddevice::~cPluginDvbhddevice(){}

bool cPluginDvbhddevice::Initialize()
{
    mIsUserInactive = true;
    dsyslog("cPluginDvbhddevice::Initialize");
  static uint32_t SubsystemIds[] = {
    0x13C23009, // Technotrend S2-6400 HDFF development samples
    0x13C2300A, // Technotrend S2-6400 HDFF production version
    0x00000000
    };
 
  for (int Adapter = 0; Adapter<MAXDVBDEVICES; Adapter++)
  {
         cString FileName;
         cReadLine ReadLine;
         FILE *f = NULL;
         uint32_t SubsystemId = 0;
         FileName = cString::sprintf("/sys/class/dvb/dvb%d.frontend0/device/subsystem_vendor", Adapter);
         if ((f = fopen(FileName, "r")) != NULL) {
            if (char *s = ReadLine.Read(f))
               SubsystemId = strtoul(s, NULL, 0) << 16;
            fclose(f);
            }
         FileName = cString::sprintf("/sys/class/dvb/dvb%d.frontend0/device/subsystem_device", Adapter);
         if ((f = fopen(FileName, "r")) != NULL) {
            if (char *s = ReadLine.Read(f))
               SubsystemId |= strtoul(s, NULL, 0);
            fclose(f);
            }
         for (uint32_t *sid = SubsystemIds; *sid; sid++) {
             if (*sid == SubsystemId) {
                FileName = cString::sprintf("/dev/dvb/adapter%d/osd0", Adapter);
                int fd = open(FileName, O_RDWR);
                if (fd != -1) { //TODO treat the second path of the S2-6400 as a budget device
                   close(fd);
                   dsyslog("creating cDvbHdFfDevice");
                   new cDvbHdFfDevice(Adapter, 0);           
                   return true;
                   }
                }
             }   
  }

  return false;
}

void cPluginDvbhddevice::MainThreadHook(void)
{
    bool isUserInactive = ShutdownHandler.IsUserInactive();
    if (isUserInactive != mIsUserInactive) 
    {
        mIsUserInactive = isUserInactive;
        if (gHdffSetup.CecEnabled && gHdffSetup.CecTvOn)
        {
            HDFF::cHdffCmdIf * hdffCmdIf = cDvbHdFfDevice::GetHdffCmdHandler();
            if (!mIsUserInactive)
            {
                hdffCmdIf->CmdHdmiSendCecCommand(HDFF_CEC_COMMAND_TV_ON);
            }
        }
    }
}

const char *cPluginDvbhddevice::MainMenuEntry(void)
{
  return gHdffSetup.HideMainMenu ? NULL : MAINMENUENTRY;
}

cOsdObject *cPluginDvbhddevice::MainMenuAction(void)
{
  return new cHdffMenu(cDvbHdFfDevice::GetHdffCmdHandler());
}

cMenuSetupPage *cPluginDvbhddevice::SetupMenu(void)
{
  return new cHdffSetupPage(cDvbHdFfDevice::GetHdffCmdHandler());
}

bool cPluginDvbhddevice::SetupParse(const char *Name, const char *Value)
{
  return gHdffSetup.SetupParse(Name, Value);
}

VDRPLUGINCREATOR(cPluginDvbhddevice); // Don't touch this!
